@@ -1,5 +1,6 @@
 """Pseudo tag and node definition for template literals."""
 
+from typing import Optional
 from typing import TextIO
 
 from liquid.ast import Node
@@ -25,8 +26,10 @@ class LiteralNode(Node):
     def __repr__(self) -> str:  # pragma: no cover
         return f"LiteralNode(tok={self.tok})"
 
-    def render_to_output(self, context: Context, buffer: TextIO):
+    # pylint: disable=useless-return
+    def render_to_output(self, context: Context, buffer: TextIO) -> Optional[bool]:
         buffer.write(self.tok.value)
+        return None
 
 
 class Literal(Tag):
