@@ -38,9 +38,9 @@ TAG_ENDUNLESS = sys.intern("endunless")
 TAG_ELSIF = sys.intern("elsif")
 TAG_ELSE = sys.intern("else")
 
-ENDUNLESSBLOCK = (TAG_ENDUNLESS, TAG_ELSIF, TAG_ELSE, TOKEN_EOF)
-ENDELSIFBLOCK = (TAG_ENDUNLESS, TAG_ELSIF, TAG_ELSE)
-ENDUNLESSELSEBLOCK = (TAG_ENDUNLESS,)
+ENDUNLESSBLOCK = frozenset((TAG_ENDUNLESS, TAG_ELSIF, TAG_ELSE, TOKEN_EOF))
+ENDELSIFBLOCK = frozenset((TAG_ENDUNLESS, TAG_ELSIF, TAG_ELSE))
+ENDUNLESSELSEBLOCK = frozenset((TAG_ENDUNLESS,))
 
 
 class UnlessNode(Node):
