@@ -100,44 +100,32 @@ cases = [
     ),
     Case(
         description="forloop length with limit",
-        template=(
-            r"{% for tag in product.tags limit:3 %}"
-            r"{{ forloop.length }} "
-            r"{% endfor %}"
-        ),
+        template=r"{% for tag in tags limit:3 %}{{ forloop.length }} {% endfor %}",
         expect="3 3 3 ",
         globals={
-            "product": {
-                "tags": [
-                    "sports",
-                    "garden",
-                    "sports",
-                    "garden",
-                    "sports",
-                    "garden",
-                ]
-            }
+            "tags": [
+                "sports",
+                "garden",
+                "home",
+                "diy",
+                "motoring",
+                "fashion",
+            ]
         },
     ),
     Case(
         description="forloop length with offset",
-        template=(
-            r"{% for tag in product.tags offset:3 %}"
-            r"{{ forloop.length }} "
-            r"{% endfor %}"
-        ),
+        template=r"{% for tag in tags offset:3 %}{{ forloop.length }} {% endfor %}",
         expect="3 3 3 ",
         globals={
-            "product": {
-                "tags": [
-                    "sports",
-                    "garden",
-                    "sports",
-                    "garden",
-                    "sports",
-                    "garden",
-                ]
-            }
+            "tags": [
+                "sports",
+                "garden",
+                "home",
+                "diy",
+                "motoring",
+                "fashion",
+            ]
         },
     ),
     Case(
@@ -257,13 +245,13 @@ cases = [
         description="continue a loop",
         template=(
             r"{% for item in array limit: 3 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in array offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 a3 b4 b5 b6 ",
         globals={"array": [1, 2, 3, 4, 5, 6]},
     ),
     Case(
@@ -271,13 +259,13 @@ cases = [
         template=(
             r"{% assign nums = (1..5) %}"
             r"{% for item in nums limit: 3 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in nums offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 ",
+        expect="a1 a2 a3 b4 b5 ",
     ),
     Case(
         description="continue a loop over a changing array",
@@ -311,13 +299,13 @@ cases = [
         description="nothing to continue from",
         template=(
             r"{% for item in array %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in array offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 a3 a4 a5 a6 ",
         globals={"array": [1, 2, 3, 4, 5, 6]},
     ),
     Case(
@@ -330,113 +318,113 @@ cases = [
         description="continue from a limit that is greater than length",
         template=(
             r"{% for item in array limit: 99 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in array offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 a3 a4 a5 a6 ",
         globals={"array": [1, 2, 3, 4, 5, 6]},
     ),
     Case(
         description="continue from a range expression",
         template=(
             r"{% for item in (1..6) limit: 3 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 a3 b4 b5 b6 ",
         globals={"array": [1, 2, 3, 4, 5, 6]},
     ),
     Case(
         description="offset continue twice with limit",
         template=(
             r"{% for item in (1..6) limit: 2 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) limit: 2 offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"c{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 b3 b4 c5 c6 ",
     ),
     Case(
         description="offset continue twice with changing limit",
         template=(
             r"{% for item in (1..6) limit: 2 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) limit: 3 offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"c{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 b3 b4 b5 c6 ",
     ),
     Case(
         description="offset continue twice with no second limit",
         template=(
             r"{% for item in (1..6) limit: 2 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"c{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 4 5 6 ",
+        expect="a1 a2 b3 b4 b5 b6 ",
     ),
     Case(
         description="offset continue from a broken loop",
         template=(
-            r"{% for item in (1..6) %}"
+            r"{% for item in (1..6) limit: 4 %}"
             r"{% if item == 3 %}{% break %}{% endif %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 ",
+        expect="a1 a2 b5 b6 ",
     ),
     Case(
         description="offset continue from a broken loop with preceding limit",
         template=(
             r"{% for item in (1..6) limit: 3 %}"
-            r"{{ item }} "
+            r"a{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) %}"
             r"{% if item == 3 %}{% break %}{% endif %}"
-            r"{{ item }} "
+            r"b{{ item }} "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} "
+            r"c{{ item }} "
             r"{% endfor %}"
         ),
-        expect="1 2 3 1 2 ",
+        expect="a1 a2 a3 b1 b2 ",
     ),
     Case(
         description="offset continue forloop length",
         template=(
             r"{% for item in (1..6) limit: 2 %}"
-            r"{{ item }} - {{ forloop.length }}, "
+            r"a{{ item }} - {{ forloop.length }}, "
             r"{% endfor %}"
             r"{% for item in (1..6) offset: continue %}"
-            r"{{ item }} - {{ forloop.length }}, "
+            r"b{{ item }} - {{ forloop.length }}, "
             r"{% endfor %}"
         ),
-        expect="1 - 2, 2 - 2, 3 - 4, 4 - 4, 5 - 4, 6 - 4, ",
+        expect="a1 - 2, a2 - 2, b3 - 4, b4 - 4, b5 - 4, b6 - 4, ",
     ),
     Case(
         description="parentloop is normally undefined",
@@ -494,7 +482,7 @@ cases = [
         description="loop over an existing range object",
         template=(
             r"{% assign foo = (1..3) %}"
-            r"{{ foo }}"
+            r"{{ foo | join: '#' }}"
             r"{% for i in foo %}"
             r"{{ i }}"
             r"{% endfor %}"
@@ -502,7 +490,7 @@ cases = [
             r"{{ i }}"
             r"{% endfor %}"
         ),
-        expect="1..3123123",
+        expect="1#2#3123123",
         globals={},
     ),
     Case(
@@ -511,4 +499,48 @@ cases = [
         expect="2345",
         globals={},
     ),
+    Case(
+        description="share outer scope",
+        template=(
+            r"{% assign foo = 'hello' %}"
+            r"{% for x in (1..3) %}"
+            r"{% assign foo = x %}"
+            r"{% endfor %}"
+            r"{{ foo }}"
+        ),
+        expect="3",
+        globals={},
+    ),
+    Case(
+        description="offset and limit",
+        template=r"{% for tag in tags limit: 3 offset: 1 %}{{ tag }} {% endfor %}",
+        expect="garden home diy ",
+        globals={"tags": ["sports", "garden", "home", "diy", "motoring", "fashion"]},
+    ),
+    Case(
+        description="first and last with an offset and limit",
+        template=(
+            r"{% for tag in tags limit: 2 offset: 1 %}"
+            r"{{ tag }} {{ forloop.first }} {{ forloop.last }} "
+            r"{% endfor %}"
+        ),
+        expect="garden true false home false true ",
+        globals={"tags": ["sports", "garden", "home", "diy", "motoring", "fashion"]},
+    ),
+    Case(
+        description="first and last with offset continue",
+        template=(
+            r"{% for tag in product.tags limit: 1 %}"
+            r"{% endfor %}"
+            r"{% for tag in product.tags offset: continue %}"
+            r"{{ forloop.first }} {{ forloop.last }} "
+            r"{% endfor %}"
+        ),
+        expect="true false false false false false false false false true ",
+        globals={
+            "product": {
+                "tags": ["sports", "garden", "home", "diy", "motoring", "fashion"]
+            }
+        },
+    ),
 ]
