@@ -1,4 +1,5 @@
 """Test miscellaneous filter functions."""
+# pylint: disable=too-many-public-methods,too-many-lines,missing-class-docstring
 import datetime
 import unittest
 
@@ -115,7 +116,7 @@ class MiscFilterTestCase(unittest.TestCase):
                 val=1,
                 args=[],
                 kwargs={},
-                expect=FilterArgumentError,
+                expect=0,
             ),
             Case(
                 description="undefined left value",
@@ -199,14 +200,7 @@ class MiscFilterTestCase(unittest.TestCase):
                 val=None,
                 args=[],
                 kwargs={},
-                expect=FilterArgumentError,
-            ),
-            Case(
-                description="not an array or string",
-                val={},
-                args=[],
-                kwargs={},
-                expect=FilterArgumentError,
+                expect="",
             ),
             Case(
                 description="false returns default",
@@ -311,7 +305,21 @@ class MiscFilterTestCase(unittest.TestCase):
                 val="March 14, 2016",
                 args=[self.env.undefined("test")],
                 kwargs={},
-                expect=FilterArgumentError,
+                expect="March 14, 2016",
+            ),
+            Case(
+                description="special 'now' value",
+                val="now",
+                args=["%Y"],
+                kwargs={},
+                expect=datetime.datetime.now().strftime("%Y"),
+            ),
+            Case(
+                description="special 'today' value",
+                val="today",
+                args=["%Y"],
+                kwargs={},
+                expect=datetime.datetime.now().strftime("%Y"),
             ),
         ]
 
